<?php
/**
 * This file is part of the wp-forecast plugin for WordPress
 *
 * Copyright 2019,2020  Hans Matzen  (email : webmaster at tuxlog dot de)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @package wp-forecast
 */

if ( ! function_exists( 'darksky_get_weather' ) ) {
	/**
	 * Build the url from the parameters and fetch the weather-data.
	 * Rreturn it as one long string.
	 *
	 * @param string $baseuri the accuweather URI.
	 * @param string $apikey  the darksyk apikey.
	 * @param string $lat     the latitutde of the location.
	 * @param string $lon     the longitude of the location.
	 * @param string $metric  which units to use.
	 */
	function darksky_get_weather( $baseuri, $apikey, $lat, $lon, $metric ) {

		if ( '1' == $metric ) {
			$metric = 'si';
		} else {
			$metric = 'us';
		}

		// check parms.
		if ( '' == trim( $apikey ) || '' == trim( $lat ) || '' == trim( $lon ) ) {
			return array();
		}
		$url1  = $baseuri . $apikey . '/' . $lat . ',' . $lon . '/?exclude=minutely,hourly';
		$url1 .= '&units=' . $metric . '&lang=en';
		// Open the file and decode it.
		// $file1 = file_get_contents($url1, false);//, $context);.
		$data = wp_remote_get( $file1['body'], true );

		return $data;
	}
}


if ( ! function_exists( 'darksky_get_data' ) ) {
	/**
	 * Extract the weather data from json object returned from darksyk.
	 *
	 * @param array $weather_array  the array to store the weather.
	 * @param array $wpf_vars       the parameters of the weather widget.
	 */
	function darksky_get_data( $weather_array, $wpf_vars ) {
		$w = array();

		if ( '1' == $wpf_vars['metric'] ) {
			$w['un_temp']  = 'C';
			$w['un_dist']  = 'km';
			$w['un_speed'] = 'm/s';
			$w['un_pres']  = 'mb';
			$w['un_prec']  = 'mm';
		} else {
			$w['un_temp']  = 'F';
			$w['un_dist']  = 'mi';
			$w['un_speed'] = 'mph';
			$w['un_pres']  = 'mb';
			$w['un_prec']  = 'in';
		}

		if ( ! isset( $weather_array['currently'] ) || ! isset( $weather_array['daily'] ) ) {
			$w['failure'] = 'No DarkSky data available';
			return $w;
		}

		$w['lat']      = $weather_array['latitude'];
		$w['lon']      = $weather_array['longitude'];
		$w['time']     = $weather_array['currently']['time'];
		$w['timezone'] = $weather_array['timezone'];
		$mtz           = new DateTimeZone( $w['timezone'] );

		// current conditions.
		$w['pressure']      = $weather_array['currently']['pressure'];
		$w['temperature']   = round( $weather_array['currently']['temperature'], 0 );
		$w['realfeel']      = round( $weather_array['currently']['apparentTemperature'], 0 );
		$w['humidity']      = $weather_array['currently']['humidity'];
		$w['weathertext']   = $weather_array['currently']['summary'];
		$w['weathericon']   = $weather_array['currently']['icon'];
		$w['wgusts']        = $weather_array['currently']['windGust'];
		$w['windspeed']     = $weather_array['currently']['windSpeed'];
		$w['winddirection'] = $weather_array['currently']['windBearing'];
		$w['uvindex']       = $weather_array['currently']['uvIndex'];

		$w['precipProbability'] = round( $weather_array['currently']['precipProbability'] * 100, 0 );
		$w['precipIntensity']   = round( $weather_array['currently']['precipIntensity'], 1 );
		if ( 0 != $w['precipIntensity'] ) {
			$w['precipType'] = $weather_array['currently']['precipType'];
		} else {
			$w['precipType'] = '';
		}
		if ( 'snow' == $w['precipType'] ) {
			$w['precipAccumulation'] = round( $weather_array['currently']['precipAccumulation'], 1 );
		} else {
			$w['precipAccumulation'] = '';
		}

		// sunset sunrise.
		$sr = new DateTime();
		$sr->setTimezone( $mtz );
		$sr->setTimestamp( $weather_array['daily']['data']['0']['sunriseTime'] );
		$w['sunrise'] = $sr->format( 'H:i' );

		$ss = new DateTime();
		$ss->setTimezone( $mtz );
		$ss->setTimestamp( $weather_array['daily']['data']['0']['sunsetTime'] );
		$w['sunset'] = $ss->format( 'H:i' );

		// forecast.
		for ( $i = 0;$i <= 7;$i++ ) {

			$j   = $i + 1;
			$odt = new DateTime();
			$odt->setTimezone( $mtz );

			$w[ 'fc_obsdate_' . $j ]      = $weather_array['daily']['data'][ $i ]['time'] + $odt->getOffset();
			$w[ 'fc_dt_short_' . $j ]     = $weather_array['daily']['data'][ $i ]['summary'];
			$w[ 'fc_dt_icon_' . $j ]      = $weather_array['daily']['data'][ $i ]['icon'];
			$w[ 'fc_dt_htemp_' . $j ]     = round( $weather_array['daily']['data'][ $i ]['temperatureHigh'], 0 );
			$w[ 'fc_dt_ltemp_' . $j ]     = round( $weather_array['daily']['data'][ $i ]['temperatureHigh'], 0 );
			$w[ 'fc_dt_windspeed_' . $j ] = $weather_array['daily']['data'][ $i ]['windSpeed'];
			$w[ 'fc_dt_winddir_' . $j ]   = $weather_array['daily']['data'][ $i ]['windBearing'];
			$w[ 'fc_dt_wgusts_' . $j ]    = $weather_array['daily']['data'][ $i ]['windGust'];
			$w[ 'fc_dt_maxuv_' . $j ]     = $weather_array['daily']['data'][ $i ]['uvIndex'];
			$w[ 'fc_nt_icon_' . $j ]      = $weather_array['daily']['data'][ $i ]['icon'];
			$w[ 'fc_nt_htemp_' . $j ]     = round( $weather_array['daily']['data'][ $i ]['temperatureLow'], 0 );
			$w[ 'fc_nt_ltemp_' . $j ]     = round( $weather_array['daily']['data'][ $i ]['temperatureLow'], 0 );
			$w[ 'fc_nt_windspeed_' . $j ] = $weather_array['daily']['data'][ $i ]['windSpeed'];
			$w[ 'fc_nt_winddir_' . $j ]   = $weather_array['daily']['data'][ $i ]['windBearing'];
			$w[ 'fc_nt_wgusts_' . $j ]    = $weather_array['daily']['data'][ $i ]['windGust'];
			$w[ 'fc_nt_maxuv_' . $j ]     = $weather_array['daily']['data'][ $i ]['uvIndex'];

			$w[ 'fc_dt_precipProbability' . $j ] = round( $weather_array['daily']['data'][ $i ]['precipProbability'] * 100, 0 );
			$w[ 'fc_dt_precipIntensity' . $j ]   = round( $weather_array['daily']['data'][ $i ]['precipIntensity'], 1 );
			if ( 0 != $w[ 'fc_dt_precipIntensity' . $j ] ) {
				$w[ 'fc_dt_precipType' . $j ] = $weather_array['daily']['data'][ $i ]['precipType'];
			} else {
				$w[ 'fc_dt_precipType' . $j ] = '';
			}
			if ( 'snow' == $w[ 'fc_dt_precipType' . $j ] ) {
				$w[ 'fc_dt_precipAccumulation' . $j ] = round( $weather_array['daily']['data'][ $i ]['precipAccumulation'], 1 );
			} else {
				$w[ 'fc_dt_precipAccumulation' . $j ] = '';
			}
		}

		// fill failure anyway.
		$w['failure'] = ( isset( $w['failure'] ) ? $w['failure'] : '' );

		return $w;
	}
}

if ( ! function_exists( 'darksky_forecast_data' ) ) {
	/**
	 * Returns the widget data as an array one line per item.
	 *
	 * @param string $wpfcid            the widget id.
	 * @param string $language_override the lanugage iso code.
	 */
	function darksky_forecast_data( $wpfcid = 'A', $language_override = null ) {

		$wpf_vars = get_wpf_opts( $wpfcid );
		if ( ! empty( $language_override ) ) {
			$wpf_vars['wpf_language'] = $language_override;
		}

		$w = maybe_unserialize( wpf_get_option( 'wp-forecast-cache' . $wpfcid ) );

		// get translations.
		global $wpf_lang_dict;
		include WP_PLUGIN_DIR . '/wp-forecast/widgetlang/wp-forecast-' . strtolower( str_replace( '_', '-', $wpf_vars['wpf_language'] ) ) . '.php';
		$wpf_lang_dict[ $wpf_vars['wpf_language'] ] = $wpf_lang;

		// --------------------------------------------------------------
		// calc values for current conditions.
		if ( isset( $w['failure'] ) && '' != $w['failure'] ) {
			return array( 'failure' => $w['failure'] );
		}

		$w['servicelink'] = 'https://darksky.net/forecast/' . $w['lat'] . ',' . $w['lon'] . '/si12/en';
		$w['copyright']   = '<a href="https://darksky.net/poweredby/">&copy; ' . gmdate( 'Y' ) . ' Powered by Dark Sky</a>';

		// next line is for compatibility.
		$w['acculink']  = $w['servicelink'];
		$w['location']  = $wpf_vars['locname'];
		$w['locname']   = $w['location'];
		$w['shorttext'] = wpf__( $w['weathertext'], $wpf_vars['wpf_language'] );

		$tz           = new DateTimeZone( $w['timezone'] );
		$w['gmtdiff'] = $tz->getOffset( new DateTime() );

		// $lt = time() - date("Z"); // this is the GMT.
		// $ct  = $lt + (3600 * ($w['gmtdiff'])); // local time.

		$ct = current_time( 'U' );
		$ct = $ct + $wpf_vars['timeoffset'] * 60; // add or subtract time offset.

		$w['blogdate'] = date_i18n( $wpf_vars['fc_date_format'], $ct );
		$w['blogtime'] = date_i18n( $wpf_vars['fc_time_format'], $ct );

		// get date/time from darksky.
		$ct            = $w['time'] + $w['gmtdiff'];
		$w['accudate'] = date_i18n( $wpf_vars['fc_date_format'], $ct );
		$w['accutime'] = date_i18n( $wpf_vars['fc_time_format'], $ct );

		$ico_arr = array(
			'clear-day'           => '01',
			'clear-night'         => '33',
			'rain'                => '12',
			'snow'                => '22',
			'sleet'               => '29',
			'wind'                => '32',
			'fog'                 => '11',
			'cloudy'              => '06',
			'partly-cloudy-day'   => '04',
			'partly-cloudy-night' => '38',
			'hail'                => '25',
			'thunderstorm'        => '15',
			'tornado'             => '32',
		);
		$ico     = '01';
		if ( isset( $ico_arr[ $w['weathericon'] ] ) ) {
			$ico = $ico_arr[ $w['weathericon'] ];
		}
		$iconfile      = find_icon( $ico );
		$w['icon']     = 'icons/' . $iconfile;
		$w['iconcode'] = $ico;

		$w['temperature'] = $w['temperature'] . '&deg;' . $w['un_temp'];
		$w['realfeel']    = $w['realfeel'] . '&deg;' . $w['un_temp'];
		$w['humidity']    = round( $w['humidity'] * 100, 0 );

		// workaround different pressure values returned by accuweather.
		$press = round( $w['pressure'], 0 );
		if ( strlen( $press ) == 3 && substr( $press, 0, 1 ) == '1' ) {
			$press = $press * 10;
		}
		$w['pressure']     = $press . ' ' . $w['un_pres'];
		$w['humidity']     = round( $w['humidity'], 0 );
		$w['windspeed']    = windstr( $wpf_vars['metric'], $w['windspeed'], $wpf_vars['windunit'] );
		$w['winddir']      = translate_winddir_degree( $w['winddirection'], 'wp-forecast_' . $wpf_vars['wpf_language'] );
		$w['winddir_orig'] = str_replace( 'O', 'E', $w['winddir'] );
		$w['windgusts']    = windstr( $wpf_vars['metric'], $w['wgusts'], $wpf_vars['windunit'] );

		// calc values for forecast.
		for ( $i = 1; $i < 8; $i++ ) {
			// daytime forecast.
			$w[ 'fc_obsdate_' . $i ] = date_i18n( $wpf_vars['fc_date_format'], $w[ 'fc_obsdate_' . $i ] );
			$ico                     = '01';
			if ( isset( $ico_arr[ $w[ 'fc_dt_icon_' . $i ] ] ) ) {
				$ico = $ico_arr[ $w[ 'fc_dt_icon_' . $i ] ];
			}
			$iconfile                        = find_icon( $ico );
			$w[ 'fc_dt_icon_' . $i ]         = 'icons/' . $iconfile;
			$w[ 'fc_dt_iconcode_' . $i ]     = $ico;
			$w[ 'fc_dt_desc_' . $i ]         = wpf__( $ico, $wpf_vars['wpf_language'] );
			$w[ 'fc_dt_htemp_' . $i ]        = $w[ 'fc_dt_htemp_' . $i ] . '&deg;' . $w['un_temp'];
			$wstr                            = windstr( $wpf_vars['metric'], $w[ 'fc_dt_windspeed_' . $i ], $wpf_vars['windunit'] );
			$w[ 'fc_dt_windspeed_' . $i ]    = $wstr;
			$w[ 'fc_dt_winddir_' . $i ]      = translate_winddir_degree( $w[ 'fc_dt_winddir_' . $i ], 'wp-forecast_' . $wpf_vars['wpf_language'] );
			$w[ 'fc_dt_winddir_orig_' . $i ] = str_replace( 'O', 'E', $w[ 'fc_dt_winddir_' . $i ] );
			$w[ 'fc_dt_wgusts_' . $i ]       = windstr( $wpf_vars['metric'], $w[ 'fc_dt_wgusts_' . $i ], $wpf_vars['windunit'] );
			$w[ 'fc_dt_maxuv_' . $i ]        = $w[ 'fc_dt_maxuv_' . $i ];

			// nighttime forecast.
			$ico = '01';
			if ( isset( $ico_arr[ $w[ 'fc_nt_icon_' . $i ] ] ) ) {
				$ico = $ico_arr[ $w[ 'fc_nt_icon_' . $i ] ];
			}
			$iconfile                        = find_icon( $ico );
			$w[ 'fc_nt_icon_' . $i ]         = 'icons/' . $iconfile;
			$w[ 'fc_nt_iconcode_' . $i ]     = $ico;
			$w[ 'fc_nt_desc_' . $i ]         = wpf__( $ico, $wpf_vars['wpf_language'] );
			$w[ 'fc_nt_ltemp_' . $i ]        = $w[ 'fc_nt_ltemp_' . $i ] . '&deg;' . $w['un_temp'];
			$wstr                            = windstr( $wpf_vars['metric'], $w[ 'fc_nt_windspeed_' . $i ], $wpf_vars['windunit'] );
			$w[ 'fc_nt_windspeed_' . $i ]    = $wstr;
			$w[ 'fc_nt_winddir_' . $i ]      = translate_winddir_degree( $w[ 'fc_nt_winddir_' . $i ], 'wp-forecast_' . $wpf_vars['wpf_language'] );
			$w[ 'fc_nt_winddir_orig_' . $i ] = str_replace( 'O', 'E', $w[ 'fc_nt_winddir_' . $i ] );
			$w[ 'fc_nt_wgusts_' . $i ]       = windstr( $wpf_vars['metric'], $w[ 'fc_nt_wgusts_' . $i ], $wpf_vars['windunit'] );
			$w[ 'fc_nt_maxuv_' . $i ]        = $w[ 'fc_nt_maxuv_' . $i ];
		}

		return $w;
	}
}
